unit FChild;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Rtti, System.Classes,
  System.Variants, FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Layouts,
  FMX.TabControl, FMX.ScrollBox, FMX.graphics,
  FMX.Memo, FMX.Controls.Presentation, FMX.Objects, FMX.Grid.Style, FMX.Grid;

type
  TF_Child = class(TForm)
    mTxt: TMemo;
    ColorLine: TRectangle;
    mLines: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure mTxtChange(Sender: TObject);
    procedure mTxtChangeTracking(Sender: TObject);
    procedure mTxtViewportPositionChange(Sender: TObject;
      const OldViewportPosition, NewViewportPosition: TPointF;
      const ContentSizeChanged: Boolean);
    procedure mTxtPainting(Sender: TObject; Canvas: TCanvas;
      const ARect: TRectF);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    TheTabItem: TTabItem;
    UnicodeType: string;
    BUnicode: Boolean;
    HasChanged: Boolean;
    TheFileName: String;
    SizeOnDisk: String;
    IsNew: Boolean;
  end;

var
  F_Child: TF_Child;
  ActiveMemo: TMemo = NIL;
  ActiveChild: TF_Child;
  FileNr: Integer = 0;
  WinNr: Integer = 0;
  ActFile: string;
  slChilds: TStringList;

implementation

{$R *.fmx}
{$R *.Macintosh.fmx MACOS}

uses
  FMain;

procedure TF_Child.FormCreate(Sender: TObject);
var
  xx: Integer;
begin
  {$IFDEF Linux}
  ColorLine.Opacity := 0.5;
  {$ENDIF}

  TheTabItem := TTabItem.Create(F_Main.TabControl1);
  TheTabItem.Visible := True;
  inc (FileNr);
  inc (WinNr);

  F_Main.TabControl1.AddObject(TheTabItem);
  TheTabItem.Text := 'Text' + intToStr (FileNr) + '.txt';

  mLines.Parent := TheTabItem;
  mLines.Font.Size := 11;
  mLines.Font.Family := mTxt.Font.Family;

  mTxt.Lines.Capacity := 100000;
  mLines.Lines.Capacity := 100000;

  mTxt.Parent := TheTabItem;
  mTxt.Font.Size := 11;
  mTxt.Tag := WinNr;
  TheTabItem.Tag := WinNr;
  slChilds.AddObject(intToStr (winnr), self);
  F_Main.TabControl1.ActiveTab := TheTabItem;
  Self.Tag := WinNr;
  ActiveMemo := mTxt;
  ActiveChild := Self;
  IsNew := True;
  ActFile := 'Text' + intToStr (FileNr) + '.txt';
end;

procedure TF_Child.mTxtChange(Sender: TObject);
begin
  HasChanged := True;
end;

procedure TF_Child.mTxtChangeTracking(Sender: TObject);
var
  c, L: Integer;
begin
  HasChanged := True;

  mLines.Lines.BeginUpdate;

  try

    if mLines.Lines.Count < mTxt.Lines.Count then begin
      c := mLines.Lines.Count;
      for L := c+1 to mTxt.Lines.Count do begin
        mLines.Lines.Add(' ' + L.ToString);
      end;
      exit;
    end;

    if mLines.Lines.Count > mTxt.Lines.Count then begin
      c := mLines.Lines.Count-1;
      for L := c downto mTxt.Lines.Count do begin
        mLines.Lines.delete (L);
      end;
      exit;
    end;

  finally
    mLines.Lines.EndUpdate;
  end;
end;

procedure TF_Child.mTxtPainting(Sender: TObject; Canvas: TCanvas;
  const ARect: TRectF);
begin
  if (ColorLine.Position.Y <> TMemo(Sender).Caret.Pos.Y) then begin
    ColorLine.Position.Y := TMemo (Sender).Caret.Pos.Y;
  end;
end;

procedure TF_Child.mTxtViewportPositionChange(Sender: TObject;
  const OldViewportPosition, NewViewportPosition: TPointF;
  const ContentSizeChanged: Boolean);
var
  vp: TPointF;
begin
  mLines.ViewportPosition := NewViewportPosition;

  if mTxt.HScrollBar.Visible then begin
    mLines.Margins.Bottom := mTxt.HScrollBar.height+1;
  end else begin
    mLines.Margins.Bottom := 0;
  end;

end;

Initialization
  slChilds := TStringList.Create;
end.
